/*
 * Copyright (C) 2019 Qunar, Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package qunar.tc.bistoury.instrument.client.classpath;

import com.google.common.base.Strings;
import com.google.common.collect.ImmutableList;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.ServiceLoader;

/**
 * @author zhenyu.nie created on 2019 2019/2/19 20:03
 */
public class DefaultAppClassPathSupplier implements AppClassPathSupplier {

    private final AppClassPathSupplier supplier;

    public DefaultAppClassPathSupplier(AppLibClassSupplier appLibClassSupplier) {
        Class<?> appLibClass = appLibClassSupplier.get();
        String libJarPath = appLibClass
                .getProtectionDomain()
                .getCodeSource()
                .getLocation()
                .getPath();
        String appLibPath = new File(libJarPath).getParentFile().getAbsolutePath();

        String appSourcePath = System.getProperty("bistoury.app.classes.path");
        if (!Strings.isNullOrEmpty(appSourcePath)) {
            supplier = new SettableAppClassPathSupplier(ImmutableList.of(appLibPath, appSourcePath));
        } else {
            Iterator<AppClassPathSupplierFactory> factoryIterator = ServiceLoader.load(AppClassPathSupplierFactory.class).iterator();
            if (factoryIterator.hasNext()) {
                supplier = factoryIterator.next().create(appLibPath);
            } else {
                supplier = new WebAppClassPathSupplier(appLibPath);
            }
        }
    }

    @Override
    public List<String> get() {
        return supplier.get();
    }
}
